@@ -36,17 +36,21 @@
 #   <http://www.gnu.org/licenses/>.
 ###############################################################################
 """
-from django.utils.translation import ugettext_lazy as _
-from django.utils.translation import ugettext
-from django.template import RequestContext
-from django.shortcuts import render_to_response
-import json as jsonlib
 from django.contrib.auth import authenticate, login
-from django.views.decorators.csrf import csrf_exempt
-from django.contrib.sites.models import Site
-from django.utils.termcolors import colorize
 from django.core.mail import mail_admins
-import traceback
+from django.shortcuts import render_to_response
+from django.template import RequestContext
+from django.views.decorators.csrf import csrf_exempt
+from django.utils import timezone
+from django.utils.termcolors import colorize
+from django.utils import translation
+from django.utils.translation import ugettext_lazy as _
+
+from quickapi.http import JSONResponse, MESSAGES
+from quickapi.conf import (settings, QUICKAPI_DEFINED_METHODS,
+    QUICKAPI_ONLY_AUTHORIZED_USERS, QUICKAPI_DEBUG, DEBUG, SITE_ID)
+
+import traceback, json as jsonlib, decimal
 
 try:
     # Проверка установленного в системе markdown.
@@ -58,9 +62,22 @@ else:
     markdown = lambda x: _markdown(x)
     BIT = '\n'
 
-from http import JSONResponse, MESSAGES
-from conf import QUICKAPI_DEFINED_METHODS, QUICKAPI_ONLY_AUTHORIZED_USERS,\
-                 QUICKAPI_DEBUG, DEBUG, SITE_ID
+if 'django.contrib.sites' in settings.INSTALLED_APPS:
+    from django.contrib.sites.models import Site
+    site = Site.objects.get(id=SITE_ID)
+else:
+    site = None
+
+def switch_language(request):
+    old_language = translation.get_language()
+    new_language = None
+    if getattr(settings, 'LANGUAGES', None) and \
+    'django.middleware.locale.LocaleMiddleware' in settings.MIDDLEWARE_CLASSES:
+        try:
+            new_language = translation.activate(request.LANGUAGE_CODE)
+        except:
+            translation.activate(old_language)
+    return old_language, new_language
 
 @csrf_exempt
 def test(request):
@@ -68,8 +85,10 @@ def test(request):
     data = {
         'integer': 9999,
         'float': 9999.999,
+        'decimal': decimal.Decimal('9999.999'),
         'boolean': True,
-        'string': 'String',
+        'string': _('String in your localization'),
+        'datetime': timezone.now(),
         'list': [9999, True, 'String'],
         'dict': {'a': 1, 'b': 2, 'c': 3}
     }
@@ -134,7 +153,7 @@ def get_methods(dic=QUICKAPI_DEFINED_METHODS):
 METHODS = get_methods()
 
 @csrf_exempt
-def index(request, dict_methods=None):
+def index(request, dict_methods=None, methods=None):
     """ Распределяет запросы.
         Структура запроса = {
             'method': u'Имя вызываемого метода',
@@ -145,21 +164,28 @@ def index(request, dict_methods=None):
             'user': u'имя пользователя',
             'pass': u'пароль пользователя',
         }
-        Параметр "dict_methods" может использоваться сторонними приложениями
-        для организации определённых наборов методов API.
-        По-умолчанию словарь методов определяется в переменной
+        Параметры "dict_methods" или "methods" могут использоваться
+        сторонними приложениями для организации определённых наборов
+        методов API.
+        Заметьте, что параметр "dict_methods" является устаревшим и
+        пока остаётся для совместимости. 
+        По-умолчанию словарь методов может определяться в переменной
         settings.QUICKAPI_DEFINED_METHODS главного проекта.
     """
+
+    switch_language(request)
+
     if QUICKAPI_DEBUG:
         p = '\nQUICKAPI:'
         print colorize(p, fg='blue')
         p = '\trequest.is_ajax()\t== %s' % request.is_ajax()
         print colorize(p, fg='blue')
 
-    if dict_methods is None:
-        methods = METHODS
-    else:
-        methods = get_methods(dict_methods)
+    if not methods:
+        if dict_methods is None:
+            methods = METHODS
+        else:
+            methods = get_methods(dict_methods)
 
     if request.is_ajax() or request.method == 'POST':
         try:
@@ -171,7 +197,7 @@ def index(request, dict_methods=None):
 
     # Vars for docs
     ctx = {}
-    ctx['site'] = Site.objects.get(id=SITE_ID)
+    ctx['site'] = site
     ctx['methods'] = methods.values()
     return render_to_response('quickapi/index.html', ctx,
                             context_instance=RequestContext(request,))
@@ -182,6 +208,8 @@ api = index
 def run(request, methods):
     """ Авторизует пользователя, если он не авторизован и запускает методы """
 
+    switch_language(request)
+
     is_authenticate = not request.user.is_anonymous()
     if QUICKAPI_DEBUG:
         p = '\trun as user\t\t== %s' % request.user
@@ -244,6 +272,8 @@ def run(request, methods):
     if QUICKAPI_DEBUG:
         p = '\tlogin user\t\t== %s' % request.user
         print colorize(p, fg='blue')
+        p = '\tmethod\t\t\t== %s' % method
+        print colorize(p, fg='blue')
 
     try:
         real_method = methods[method]['method']
@@ -261,6 +291,16 @@ def run(request, methods):
         if DEBUG:
             print msg
         else:
-            mail_admins('QuickAPI method error', msg +'\n\n'+ unicode(request))
-            msg = MESSAGES[500]
-        return JSONResponse(status=500, message=msg)
+            try:
+                msg = msg.decode('utf-8')
+            except:
+                pass
+            mail_admins(u'QuickAPI method error', msg +'\n\n'+ unicode(request))
+        try:
+            return JSONResponse(status=500, message=str(e).decode('utf-8'))
+        except:
+            try:
+                return JSONResponse(status=500, message=unicode(e))
+            except:
+                pass
+            return JSONResponse(status=500, message=MESSAGES[500])
